@@ -40,10 +40,12 @@
 #include "zoom-region-private.h"
 #include "GNOME_Magnifier.h"
 
+/* if you #define this, don't forget to set MAG_CLIENT_DEBUG env variable */
 #undef DEBUG_CLIENT_CALLS
 
 #ifdef DEBUG_CLIENT_CALLS
-#define DBG(a) a
+gboolean client_debug = FALSE;
+#define DBG(a) if (client_debug) { (a); }
 #else
 #define DBG(a)
 #endif
@@ -117,6 +119,7 @@ static Magnifier *_this_magnifier = NULL;
 
 static void magnifier_transform_cursor (Magnifier *magnifier);
 static void magnifier_init_cursor_set (Magnifier *magnifier, gchar *cursor_set);
+static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
 static gboolean magnifier_check_set_struts (Magnifier *magnifier);
 static gboolean magnifier_reset_struts_at_idle (gpointer data);
 static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
@@ -319,6 +322,19 @@ magnifier_free_cursor_pixels (guchar *pixels, gpointer data)
     /* XFree (data); FIXME why doesn't this work properly? */
 }
 
+#ifdef HAVE_XFIXES
+static void
+magnifier_cursor_convert_to_rgba (Magnifier *magnifier, XFixesCursorImage *cursor_image)
+{
+	int i, count = cursor_image->width * cursor_image->height;
+	for (i = 0; i < count; ++i) 
+	{
+		guint32 pixval = GUINT_TO_LE (cursor_image->pixels[i]);
+		cursor_image->pixels[i] = pixval;
+	}
+}
+#endif
+
 GdkPixbuf *
 magnifier_get_source_pixbuf (Magnifier *magnifier)
 {
@@ -328,6 +344,7 @@ magnifier_get_source_pixbuf (Magnifier *magnifier)
 	gchar s[6];
 	if (cursor_image)
 	{
+	        magnifier_cursor_convert_to_rgba (magnifier, cursor_image);
 		cursor_pixbuf = gdk_pixbuf_new_from_data ((guchar *) cursor_image->pixels, 
 							  GDK_COLORSPACE_RGB, 
 							  TRUE, 8,  
@@ -1728,6 +1745,9 @@ magnifier_init (Magnifier *magnifier)
 			       gdk_display_get_screen (magnifier->target_display, 
 						       magnifier->target_screen_num));
 	magnifier_init_cursor_set (magnifier, "default");
+#ifdef DEBUG_CLIENT_CALLS
+	client_debug = g_getenv ("MAG_CLIENT_DEBUG");
+#endif
 }
 
 GdkDrawable *
