@@ -9,6 +9,8 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+
+
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -781,25 +783,22 @@ zoom_region_unpaint_crosswire_cursor (ZoomRegion *zoom_region,
 {
 	Magnifier *magnifier = zoom_region->priv->parent;
 	GdkRectangle vline_rect, hline_rect;
-	GdkRectangle rect;
+	GdkPoint cursor_pos;
 
 	if (magnifier->crosswire_size <= 0) return;
 
-	rect = zoom_region->priv->cursor_backing_rect;
-	rect.x += magnifier->cursor_hotspot.x;
-	rect.y += magnifier->cursor_hotspot.y;
-	vline_rect.x = rect.x - magnifier->crosswire_size/2;
+	cursor_pos = zoom_region->priv->last_drawn_crosswire_pos;
+	vline_rect.x = cursor_pos.x - magnifier->crosswire_size/2;
 	vline_rect.y = clip_rect ? clip_rect->y : 0; 
 	vline_rect.width = MAX (magnifier->crosswire_size, 1);
 	vline_rect.height = clip_rect ? clip_rect->height : 4096; 
 	hline_rect.x = clip_rect ? clip_rect->x : 0; 
-	hline_rect.y = rect.y - magnifier->crosswire_size/2;
+	hline_rect.y = cursor_pos.y - magnifier->crosswire_size/2;
 	hline_rect.width = clip_rect ? clip_rect->width : 4096;
 	hline_rect.height = MAX (magnifier->crosswire_size, 1);
 
 	zoom_region_paint_pixmap (zoom_region, &vline_rect);
 	zoom_region_paint_pixmap (zoom_region, &hline_rect);
-/*	gdk_display_sync (gdk_drawable_get_display (zoom_region->priv->w->window)); */
 }
 
 static void
@@ -817,8 +816,9 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 	int x_left_clip = 0, x_right_clip = 0, y_top_clip = 0, y_bottom_clip = 0;
 	int csize;
 	
-	if (!(magnifier && GDK_IS_DRAWABLE (zoom_region->priv->w) &&
+	if (!(magnifier &&
 	      zoom_region->priv->w->window &&
+	      GDK_IS_DRAWABLE (zoom_region->priv->w->window) &&
 	      magnifier->crosswire_size > 0)) return;
 
 	if (zoom_region->priv->crosswire_gc == NULL) 
@@ -830,9 +830,9 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 
 	if (magnifier->crosswire_color == 0)
 	{
-		color.red = 0xFF;
-		color.blue = 0xFF;
-		color.green = 0xFF;
+		color.red = 0xFFFF;
+		color.blue = 0xFFFF;
+		color.green = 0xFFFF;
 		values.function = GDK_INVERT;
 	}
 	else
@@ -869,6 +869,7 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 	rect.y = zoom_region->priv->last_cursor_pos.y;
 	rect = zoom_region_view_rect_from_source_rect (zoom_region, rect);
 	if (clip_rect) gdk_gc_set_clip_rectangle (zoom_region->priv->crosswire_gc, clip_rect);
+	else gdk_gc_set_clip_rectangle (zoom_region->priv->crosswire_gc, NULL);
 
 	if (magnifier->crosswire_clip &&
 	    (cursor = magnifier_get_cursor (magnifier)))
@@ -940,6 +941,10 @@ zoom_region_paint_cursor (ZoomRegion *zoom_region,
 							   &zoom_region->viewport);
 		clip_rect = &fullscreen;
 	}
+	/* save the unclipped cursor pos for 'undrawing' the crosswire, the clipped one is no good */
+	zoom_region->priv->last_drawn_crosswire_pos.x = rect.x + magnifier->cursor_hotspot.x;
+	zoom_region->priv->last_drawn_crosswire_pos.y = rect.y + magnifier->cursor_hotspot.y;
+
 	if (gdk_rectangle_intersect (clip_rect, &rect, &intersct))
 	{
 		int width = 0, height = 0;
@@ -1106,10 +1111,9 @@ zoom_region_paint (ZoomRegion *zoom_region,
 		   GdkRectangle *area)
 {
 	GdkRectangle paint_area;
-	DEBUG_RECT ("painting", *area);
 	DEBUG_RECT ("painting (clipped)", *area);
-	zoom_region_paint_border (zoom_region, area);
 	paint_area = zoom_region_clip_to_window (zoom_region, *area);
+	zoom_region_paint_border (zoom_region, area);
 	zoom_region_paint_pixmap (zoom_region, &paint_area);
 	zoom_region_paint_cursor (zoom_region, &paint_area);
 	zoom_region_paint_crosswire_cursor (zoom_region, &paint_area);
@@ -1172,12 +1176,15 @@ zoom_region_expose_handler (GtkWindow * w,
 static void zoom_region_update_cursor (ZoomRegion *zoom_region, int dx, int dy,
 				       GdkRectangle *clip_rect)
 {
+	zoom_region_unpaint_crosswire_cursor (zoom_region, clip_rect);
+	zoom_region_unpaint_cursor (zoom_region, clip_rect);
 	zoom_region->priv->cursor_backing_rect.x += dx;
 	zoom_region->priv->cursor_backing_rect.y += dy;
-	zoom_region_unpaint_crosswire_cursor (zoom_region, NULL);
-	zoom_region_unpaint_cursor (zoom_region, clip_rect);
-	zoom_region_paint_crosswire_cursor (zoom_region, clip_rect);
+	zoom_region->priv->last_drawn_crosswire_pos.x += dx;
+	zoom_region->priv->last_drawn_crosswire_pos.y += dy;
 	zoom_region_paint_cursor (zoom_region, clip_rect);
+	zoom_region_paint_crosswire_cursor (zoom_region, clip_rect);
+	gdk_display_sync (gdk_drawable_get_display (zoom_region->priv->w->window));
 }
 
 static gboolean
@@ -1241,10 +1248,11 @@ zoom_region_scroll_fast (ZoomRegion *zoom_region, int dx, int dy,
 		processing_updates = FALSE;
 		return;
 	}
-	if (zoom_region->smooth_scroll_policy > GNOME_Magnifier_ZoomRegion_SCROLL_FAST) 
-		gdk_display_sync (gdk_drawable_get_display (window));
+	zoom_region_unpaint_crosswire_cursor (zoom_region, scroll_rect);
+	zoom_region_unpaint_cursor (zoom_region, scroll_rect);
 	gdk_window_scroll (window, dx, dy);
-	zoom_region_update_cursor (zoom_region, dx, dy, scroll_rect);
+	zoom_region_paint_cursor (zoom_region, scroll_rect);
+	zoom_region_paint_crosswire_cursor (zoom_region, scroll_rect);
 	gdk_window_process_updates (window, FALSE);
 	/* sync reduces cursor flicker, but slows things down */
 	if (zoom_region->smooth_scroll_policy > GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST)
@@ -1257,10 +1265,12 @@ zoom_region_scroll_smooth (ZoomRegion *zoom_region, int dx, int dy,
 			   GdkRectangle *expose_rect_h,
 			   GdkRectangle *expose_rect_v)
 {
-	GdkWindow *window = zoom_region->priv->w->window;
+	GdkWindow *window = NULL;
 	GdkRectangle window_rect;
 
-	if (!window)
+	if (zoom_region->priv->w && GDK_IS_DRAWABLE (zoom_region->priv->w->window))
+		window = zoom_region->priv->w->window;
+	else
 		return;
 	window_rect.x = 0;
 	window_rect.y = 0;
@@ -1409,7 +1419,22 @@ zoom_region_update_pointer (ZoomRegion *zoom_region, gboolean draw_cursor)
 			zoom_region_set_cursor_pos (zoom_region,
 						    mouse_x_return, mouse_y_return);
 			if (draw_cursor)
-				zoom_region_update_cursor (zoom_region, 0, 0, NULL);
+			{
+				GdkRectangle paint_area, *clip = NULL;
+				if (GDK_IS_DRAWABLE (zoom_region->priv->w->window))
+				{
+					gdk_drawable_get_size (
+						GDK_DRAWABLE (
+							zoom_region->priv->w->window),
+						&paint_area.width, &paint_area.height);
+					paint_area.x = 0;
+					paint_area.y = 0;
+					clip = &paint_area;
+					paint_area = zoom_region_clip_to_source (
+						zoom_region, paint_area);
+				}
+				zoom_region_update_cursor (zoom_region, 0, 0, clip);
+			}
 			return TRUE;
 		}
 	}	
@@ -2554,7 +2579,7 @@ impl_zoom_region_mark_dirty (PortableServer_Servant servant,
 		ZOOM_REGION (bonobo_object_from_servant (servant));
 	DEBUG_RECT ("mark dirty", zoom_region_rect_from_bounds (
 			    zoom_region, roi_dirty) );
-	zoom_region_update_pointer (zoom_region, FALSE);
+	zoom_region_update_pointer (zoom_region, TRUE);
 	zoom_region_queue_update (zoom_region, 
 	  zoom_region_clip_to_target (zoom_region, 
 	      zoom_region_rect_from_bounds (zoom_region, roi_dirty)));
@@ -2884,6 +2909,10 @@ zoom_region_private_init (ZoomRegionPrivate *priv)
 	priv->gdk_interp_type = GDK_INTERP_NEAREST;
 	priv->expose_handler_id = 0;
 	priv->test = FALSE;
+	priv->last_cursor_pos.x = 0;
+	priv->last_cursor_pos.y = 0;
+	priv->last_drawn_crosswire_pos.x = 0;
+	priv->last_drawn_crosswire_pos.y = 0;
 }
 
 static void
@@ -2891,7 +2920,7 @@ zoom_region_init (ZoomRegion *zoom_region)
 {
 	zoom_region_properties_init (zoom_region);
 	zoom_region->smooth_scroll_policy =
-		GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST;
+		GNOME_Magnifier_ZoomRegion_SCROLL_SMOOTH;
 	zoom_region->invert = FALSE;
 	zoom_region->cache_source = FALSE;
 	zoom_region->border_size = 0;
